; music.asm

; OUT ports
#define PORT_LCD $D     ; 1101 - bit 1 is low

; Bit flags in LCDCONTROL port
#define LCD_REG_COMMAND $4
#define LCD_REG_DATA $6

; LCD commands
#define LCD_COMMAND_INTERFACE8 $3C  ; 8-bit interface, 2-line display, 5x10 font
#define LCD_COMMAND_INTERFACE4 $2C  ; 4-bit interface, 2-line display, 5x10 font
#define LCD_COMMAND_DISPLAY $0C     ; display on, cursor off, blinking cursor off
#define LCD_COMMAND_CLEAR $01       ; clear display, home cursor
#define LCD_COMMAND_CURSOR_POS_LINE_1 $80 ; OR the desired cursor position with $80 to create a cursor position command. Line 2 begins at pos 64
#define LCD_COMMAND_CURSOR_POS_LINE_2 $C0  
#define LCD_COMMAND_CURSOR_POS_GUESS $CC

; memory locations
#define LCD_BUFFER_INDEX $010
#define LCD_CONTROL_STATE $011
#define LCD_DELAY_1 $012
#define LCD_DELAY_2 $013
#define LCD_BUFFER $020
#define TEXT_RETURN_ADDR $014

; begin music-stub.asm

; OUT ports
#define PORT_CONTROL $E ; 1110 - bit 0 is low

; memory locations
#define TONE_PERIOD_0 $000
#define TONE_PERIOD_1 $001
#define TONE_PERIOD_2 $002
#define TONE_COUNTER_0 $003
#define TONE_COUNTER_1 $004
#define TONE_COUNTER_2 $005
#define TONE_DURATION_0 $006
#define TONE_DURATION_1 $007
#define TONE_DURATION_2 $008
#define DELAY_COUNTER_0 $009
#define DELAY_COUNTER_1 $00A
#define DELAY_COUNTER_2 $00B
#define DELAY_COUNTER_3 $00C
#define RETURN_ADDR_0 $00D
#define RETURN_ADDR_1 $00E

jmp init

rest_delay:
    ; total loop is 50 instructions, 100 clock cycles
    ; first loop is 34 instructions
    lit #11
-   addi #0
    addi #-1
    jnz -
    ; 16 instructions below
    ld DELAY_COUNTER_0
    addi #-1
    st DELAY_COUNTER_0
    jc rest_delay_1
    ld DELAY_COUNTER_1
    addi #-1
    st DELAY_COUNTER_1
    jc rest_delay_2
    ld DELAY_COUNTER_2
    addi #-1
    st DELAY_COUNTER_2
    jc rest_delay_3
    ld DELAY_COUNTER_3
    addi #-1
    st DELAY_COUNTER_3
    jc rest_delay

    jmp rest_delay_return

    ; make each loop be constant, regardless of how many nibbles were checked
rest_delay_1:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
rest_delay_2:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
rest_delay_3:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    jmp rest_delay


audio_play_tone:
    ; prepare to send an LCD command
    lit #LCD_REG_COMMAND
    st LCD_CONTROL_STATE
    lit #<LCD_COMMAND_CURSOR_POS_LINE_1
    st LCD_BUFFER+1
    lit #>LCD_COMMAND_CURSOR_POS_LINE_1
    st LCD_BUFFER+0
    ; remember where to return
    lit #5
    st TEXT_RETURN_ADDR
    ; data begins at buffer index 1
    lit #1
    jmp lcd_write_buffer

textnext5:
    ; prepare to send LCD character data
    lit #LCD_REG_DATA
    st LCD_CONTROL_STATE

    ld TONE_PERIOD_2
    cmpi #10
    jl +
    addi #-9
    st LCD_BUFFER+14
    lit #<'A'
    st LCD_BUFFER+15
    jmp ++
+   st LCD_BUFFER+14
    lit #<'0'
    st LCD_BUFFER+15
++

    ld TONE_PERIOD_1
    cmpi #10
    jl +
    addi #-9
    st LCD_BUFFER+12
    lit #<'A'
    st LCD_BUFFER+13
    jmp ++
+   st LCD_BUFFER+12
    lit #<'0'
    st LCD_BUFFER+13
++

    ld TONE_PERIOD_0
    cmpi #10
    jl +
    addi #-9
    st LCD_BUFFER+10
    lit #<'A'
    st LCD_BUFFER+11
    jmp ++
+   st LCD_BUFFER+10
    lit #<'0'
    st LCD_BUFFER+11
++

    ld TONE_DURATION_2
    cmpi #10
    jl +
    addi #-9
    st LCD_BUFFER+8
    lit #<'A'
    st LCD_BUFFER+9
    jmp ++
+   st LCD_BUFFER+8
    lit #<'0'
    st LCD_BUFFER+9
++

    ld TONE_DURATION_1
    cmpi #10
    jl +
    addi #-9
    st LCD_BUFFER+6
    lit #<'A'
    st LCD_BUFFER+7
    jmp ++
+   st LCD_BUFFER+6
    lit #<'0'
    st LCD_BUFFER+7
++

    ld TONE_DURATION_0
    cmpi #10
    jl +
    addi #-9
    st LCD_BUFFER+4
    lit #<'A'
    st LCD_BUFFER+5
    jmp ++
+   st LCD_BUFFER+4
    lit #<'0'
    st LCD_BUFFER+5
++

    lit #>' '
    st LCD_BUFFER+2
    lit #<' '
    st LCD_BUFFER+3

    ld RETURN_ADDR_0
    cmpi #10
    jl +
    addi #-9
    st LCD_BUFFER+0
    lit #<'A'
    st LCD_BUFFER+1
    jmp ++
+   st LCD_BUFFER+0
    lit #<'0'
    st LCD_BUFFER+1
++

    ; remember where to return
+   lit #6
    st TEXT_RETURN_ADDR
    ; data begins at buffer index 15
    lit #15
    jmp lcd_write_buffer

audio_play_tone_cont:
textnext6:
    ; actual period in microseconds = (48 * (TONE_PERIOD + 1) + 60) / CPU_CLOCKS_PER_US
    ; TONE_PERIOD = (desired_period_us * CPU_CLOCKS_PER_US - 108) / 48
    lit #$4
    out #PORT_CONTROL

    ld TONE_PERIOD_0
    st TONE_COUNTER_0
    ld TONE_PERIOD_1
    st TONE_COUNTER_1
    ld TONE_PERIOD_2
    st TONE_COUNTER_2

    ; each pass through the loop takes 12 instructions
wait_low:
    ld TONE_COUNTER_0
    addi #-1
    st TONE_COUNTER_0
    jc period_delay_low_1
    ld TONE_COUNTER_1
    addi #-1
    st TONE_COUNTER_1
    jc period_delay_low_2
    ld TONE_COUNTER_2
    addi #-1
    st TONE_COUNTER_2
    jc wait_low
    jmp done_low

    ; make the period be constant, regardless of how many period nibbles were checked
period_delay_low_1:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
period_delay_low_2:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    jmp wait_low

done_low:
    lit #$C
    out #PORT_CONTROL
      
    ld TONE_PERIOD_0
    st TONE_COUNTER_0
    ld TONE_PERIOD_1
    st TONE_COUNTER_1
    ld TONE_PERIOD_2
    st TONE_COUNTER_2

    ; each pass through the loop takes 12 instructions
wait_high:
    ld TONE_COUNTER_0
    addi #-1
    st TONE_COUNTER_0
    jc period_delay_high_1
    ld TONE_COUNTER_1
    addi #-1
    st TONE_COUNTER_1
    jc period_delay_high_2
    ld TONE_COUNTER_2
    addi #-1
    st TONE_COUNTER_2
    jc wait_high
    jmp done_high

    ; make the period be constant, regardless of how many period nibbles were checked
period_delay_high_1:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
period_delay_high_2:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    jmp wait_high

done_high:         
    ; end of one tone period
    ld TONE_DURATION_0
    addi #-1
    st TONE_DURATION_0
    jc duration_delay_1
    ld TONE_DURATION_1
    addi #-1
    st TONE_DURATION_1
    jc duration_delay_2
    ld TONE_DURATION_2
    addi #-1
    st TONE_DURATION_2
    jc audio_play_tone_cont
          
    jmp rest_delay

    ; make the period be constant, regardless of how many duration nibbles were checked
duration_delay_1:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
duration_delay_2:
    addi #0 ; NOP
    addi #0 ; NOP
    addi #0 ; NOP
    jmp audio_play_tone_cont

; end music-stub.asm
; the code below was auto geneated by Nibbler Music Maker

#define VERSE_COUNTER $00F

init:
    ; To initialize the LCD from an unknown state, we must first set it to 8-bit mode three times. Then it can be set to 4-bit mode.

    ; prepare to send an LCD command
    lit #LCD_REG_COMMAND
    st LCD_CONTROL_STATE
    ; remember where to return 
    lit #0
    st TEXT_RETURN_ADDR
    ; zero more nibbles to read from the buffer
    lit #0
    st LCD_BUFFER_INDEX
    ; the command is INTERFACE8
    lit #<LCD_COMMAND_INTERFACE8
    jmp lcd_write_nibble

textnext0:
    ; remember where to return
    lit #0
    st RETURN_ADDR_0
    jmp lcd_long_delay

delaynext0:
    ; remember where to return
    lit #1
    st TEXT_RETURN_ADDR
    ; zero more nibbles to read from the buffer
    lit #0
    st LCD_BUFFER_INDEX
    ; the command is INTERFACE8
    lit #<LCD_COMMAND_INTERFACE8
    jmp lcd_write_nibble

textnext1:
    ; remember where to return
    lit #1
    st RETURN_ADDR_0
    jmp lcd_long_delay

delaynext1:
    ; remember where to return
    lit #2
    st TEXT_RETURN_ADDR
    ; zero more nibbles to read from the buffer
    lit #0
    st LCD_BUFFER_INDEX
    ; the command is INTERFACE8
    lit #<LCD_COMMAND_INTERFACE8
    jmp lcd_write_nibble

textnext2:
    ; remember where to return
    lit #3
    st TEXT_RETURN_ADDR
    ; zero more nibbles to read from the buffer
    lit #0
    st LCD_BUFFER_INDEX
    ; the command is INTERFACE4
    lit #<LCD_COMMAND_INTERFACE4
    jmp lcd_write_nibble

textnext3:
    ; The LCD is now in 4-bit mode, and we can send byte-wide commands as a pair of nibble, high nibble first.
    ; prepare to send an LCD command
    lit #LCD_REG_COMMAND
    st LCD_CONTROL_STATE

    lit #<LCD_COMMAND_INTERFACE4
    st LCD_BUFFER+5
    lit #>LCD_COMMAND_INTERFACE4
    st LCD_BUFFER+4
    lit #<LCD_COMMAND_DISPLAY
    st LCD_BUFFER+3
    lit #>LCD_COMMAND_DISPLAY
    st LCD_BUFFER+2
    lit #<LCD_COMMAND_CLEAR
    st LCD_BUFFER+1
    lit #>LCD_COMMAND_CLEAR
    st LCD_BUFFER+0

    ; remember where to return
    lit #4
    st TEXT_RETURN_ADDR
    ; data begins at buffer index 5
    lit #5
    jmp lcd_write_buffer

textnext4:
    ; remember where to return
    lit #2
    st RETURN_ADDR_0
    jmp lcd_long_delay

delaynext2:
lit #0
st VERSE_COUNTER
jmp begin

part:
; Play note 50 4D, freq = 293.66 Hz, period 3406 us, TONE_PERIOD $AC, duration 218750 us (64 periods = $40), break 31250 us (768 delays = $300)
lit #$0
st TONE_PERIOD_2
lit #$A
st TONE_PERIOD_1
lit #$C
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$4
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$3
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$0
st RETURN_ADDR_0
jmp audio_play_tone

next0:
; Play note 54 4F#, freq = 369.99 Hz, period 2703 us, TONE_PERIOD $88, duration 218750 us (81 periods = $51), break 31250 us (768 delays = $300)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$5
st TONE_DURATION_1
lit #$1
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$3
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$1
st RETURN_ADDR_0
jmp audio_play_tone

next1:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 218750 us (96 periods = $60), break 31250 us (768 delays = $300)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$3
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$2
st RETURN_ADDR_0
jmp audio_play_tone

next2:
; Play note 50 4D, freq = 293.66 Hz, period 3406 us, TONE_PERIOD $AC, duration 875000 us (257 periods = $101), break 125000 us (3072 delays = $C00)
lit #$0
st TONE_PERIOD_2
lit #$A
st TONE_PERIOD_1
lit #$C
st TONE_PERIOD_0
lit #$1
st TONE_DURATION_2
lit #$0
st TONE_DURATION_1
lit #$1
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$C
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$3
st RETURN_ADDR_0
jmp audio_play_tone

next3:
; Play note 62 5D, freq = 587.33 Hz, period 1703 us, TONE_PERIOD $55, duration 437500 us (257 periods = $101), break 62500 us (1536 delays = $600)
lit #$0
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$1
st TONE_DURATION_2
lit #$0
st TONE_DURATION_1
lit #$1
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$6
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$4
st RETURN_ADDR_0
jmp audio_play_tone

next4:
; Play note 62 5D, freq = 587.33 Hz, period 1703 us, TONE_PERIOD $55, duration 437500 us (257 periods = $101), break 62500 us (1536 delays = $600)
lit #$0
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$1
st TONE_DURATION_2
lit #$0
st TONE_DURATION_1
lit #$1
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$6
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$5
st RETURN_ADDR_0
jmp audio_play_tone

next5:
; Rest duration 500000 us (12288 delays = $3000)
lit #$3
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$6
st RETURN_ADDR_0
jmp rest_delay

next6:
; end of part
ld VERSE_COUNTER
addi #1
st VERSE_COUNTER
cmpi #1
jz middle
jmp conclusion

begin:
; Play note 62 5D, freq = 587.33 Hz, period 1703 us, TONE_PERIOD $55, duration 291666.7 us (171 periods = $AB), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$A
st TONE_DURATION_1
lit #$B
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$7
st RETURN_ADDR_0
jmp audio_play_tone

next7:
; Play note 66 5F#, freq = 739.99 Hz, period 1352 us, TONE_PERIOD $43, duration 291666.7 us (216 periods = $D8), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$4
st TONE_PERIOD_1
lit #$3
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$D
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$8
st RETURN_ADDR_0
jmp audio_play_tone

next8:
; Play note 69 5A, freq = 880.00 Hz, period 1137 us, TONE_PERIOD $38, duration 291666.7 us (257 periods = $101), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$3
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$1
st TONE_DURATION_2
lit #$0
st TONE_DURATION_1
lit #$1
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$9
st RETURN_ADDR_0
jmp audio_play_tone

next9:
; Play note 69 5A, freq = 880.00 Hz, period 1137 us, TONE_PERIOD $38, duration 583333.3 us (513 periods = $201), break 83333.34 us (2048 delays = $800)
lit #$0
st TONE_PERIOD_2
lit #$3
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$2
st TONE_DURATION_2
lit #$0
st TONE_DURATION_1
lit #$1
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$8
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$A
st RETURN_ADDR_0
jmp audio_play_tone

next10:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$B
st RETURN_ADDR_0
jmp audio_play_tone

next11:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$C
st RETURN_ADDR_0
jmp audio_play_tone

next12:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$D
st RETURN_ADDR_0
jmp rest_delay

next13:
; Play note 54 4F#, freq = 369.99 Hz, period 2703 us, TONE_PERIOD $88, duration 291666.7 us (108 periods = $6C), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$C
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$E
st RETURN_ADDR_0
jmp audio_play_tone

next14:
; Play note 54 4F#, freq = 369.99 Hz, period 2703 us, TONE_PERIOD $88, duration 291666.7 us (108 periods = $6C), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$C
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$0
st RETURN_ADDR_1
lit #$F
st RETURN_ADDR_0
jmp audio_play_tone

next15:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$0
st RETURN_ADDR_0
jmp rest_delay

next16:
; Play note 38 3D, freq = 146.83 Hz, period 6811 us, TONE_PERIOD $15A, duration 291666.7 us (43 periods = $2B), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$A
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$B
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$1
st RETURN_ADDR_0
jmp audio_play_tone

next17:
; Play note 38 3D, freq = 146.83 Hz, period 6811 us, TONE_PERIOD $15A, duration 291666.7 us (43 periods = $2B), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$A
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$B
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$2
st RETURN_ADDR_0
jmp audio_play_tone

next18:
; Play note 42 3F#, freq = 185.00 Hz, period 5406 us, TONE_PERIOD $113, duration 291666.7 us (54 periods = $36), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$1
st TONE_PERIOD_1
lit #$3
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$3
st TONE_DURATION_1
lit #$6
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$3
st RETURN_ADDR_0
jmp audio_play_tone

next19:
; Play note 45 3A, freq = 220.00 Hz, period 4546 us, TONE_PERIOD $E7, duration 291666.7 us (64 periods = $40), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$E
st TONE_PERIOD_1
lit #$7
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$4
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$4
st RETURN_ADDR_0
jmp audio_play_tone

next20:
; Play note 45 3A, freq = 220.00 Hz, period 4546 us, TONE_PERIOD $E7, duration 583333.3 us (128 periods = $80), break 83333.34 us (2048 delays = $800)
lit #$0
st TONE_PERIOD_2
lit #$E
st TONE_PERIOD_1
lit #$7
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$8
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$5
st RETURN_ADDR_0
jmp audio_play_tone

next21:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$6
st RETURN_ADDR_0
jmp audio_play_tone

next22:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$7
st RETURN_ADDR_0
jmp audio_play_tone

next23:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$8
st RETURN_ADDR_0
jmp rest_delay

next24:
; Play note 55 4G, freq = 392.00 Hz, period 2552 us, TONE_PERIOD $80, duration 291666.7 us (114 periods = $72), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$0
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$7
st TONE_DURATION_1
lit #$2
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$9
st RETURN_ADDR_0
jmp audio_play_tone

next25:
; Play note 55 4G, freq = 392.00 Hz, period 2552 us, TONE_PERIOD $80, duration 291666.7 us (114 periods = $72), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$0
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$7
st TONE_DURATION_1
lit #$2
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$A
st RETURN_ADDR_0
jmp audio_play_tone

next26:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$B
st RETURN_ADDR_0
jmp rest_delay

next27:
; Play note 37 3C#, freq = 138.59 Hz, period 7216 us, TONE_PERIOD $16F, duration 291666.7 us (40 periods = $28), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$F
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$C
st RETURN_ADDR_0
jmp audio_play_tone

next28:
; Play note 37 3C#, freq = 138.59 Hz, period 7216 us, TONE_PERIOD $16F, duration 291666.7 us (40 periods = $28), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$F
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$D
st RETURN_ADDR_0
jmp audio_play_tone

next29:
; Play note 40 3E, freq = 164.81 Hz, period 6068 us, TONE_PERIOD $134, duration 291666.7 us (48 periods = $30), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$3
st TONE_PERIOD_1
lit #$4
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$3
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$E
st RETURN_ADDR_0
jmp audio_play_tone

next30:
; Play note 47 3B, freq = 246.94 Hz, period 4050 us, TONE_PERIOD $CD, duration 291666.7 us (72 periods = $48), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$C
st TONE_PERIOD_1
lit #$D
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$4
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$1
st RETURN_ADDR_1
lit #$F
st RETURN_ADDR_0
jmp audio_play_tone

next31:
; Play note 47 3B, freq = 246.94 Hz, period 4050 us, TONE_PERIOD $CD, duration 583333.3 us (144 periods = $90), break 83333.34 us (2048 delays = $800)
lit #$0
st TONE_PERIOD_2
lit #$C
st TONE_PERIOD_1
lit #$D
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$8
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$0
st RETURN_ADDR_0
jmp audio_play_tone

next32:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 291666.7 us (144 periods = $90), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$1
st RETURN_ADDR_0
jmp audio_play_tone

next33:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 291666.7 us (144 periods = $90), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$2
st RETURN_ADDR_0
jmp audio_play_tone

next34:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$3
st RETURN_ADDR_0
jmp rest_delay

next35:
; Play note 55 4G, freq = 392.00 Hz, period 2552 us, TONE_PERIOD $80, duration 291666.7 us (114 periods = $72), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$0
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$7
st TONE_DURATION_1
lit #$2
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$4
st RETURN_ADDR_0
jmp audio_play_tone

next36:
; Play note 55 4G, freq = 392.00 Hz, period 2552 us, TONE_PERIOD $80, duration 291666.7 us (114 periods = $72), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$0
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$7
st TONE_DURATION_1
lit #$2
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$5
st RETURN_ADDR_0
jmp audio_play_tone

next37:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$6
st RETURN_ADDR_0
jmp rest_delay

next38:
; Play note 37 3C#, freq = 138.59 Hz, period 7216 us, TONE_PERIOD $16F, duration 291666.7 us (40 periods = $28), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$F
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$7
st RETURN_ADDR_0
jmp audio_play_tone

next39:
; Play note 37 3C#, freq = 138.59 Hz, period 7216 us, TONE_PERIOD $16F, duration 291666.7 us (40 periods = $28), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$F
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$8
st RETURN_ADDR_0
jmp audio_play_tone

next40:
; Play note 40 3E, freq = 164.81 Hz, period 6068 us, TONE_PERIOD $134, duration 291666.7 us (48 periods = $30), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$3
st TONE_PERIOD_1
lit #$4
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$3
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$9
st RETURN_ADDR_0
jmp audio_play_tone

next41:
; Play note 47 3B, freq = 246.94 Hz, period 4050 us, TONE_PERIOD $CD, duration 291666.7 us (72 periods = $48), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$C
st TONE_PERIOD_1
lit #$D
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$4
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$A
st RETURN_ADDR_0
jmp audio_play_tone

next42:
; Play note 47 3B, freq = 246.94 Hz, period 4050 us, TONE_PERIOD $CD, duration 583333.3 us (144 periods = $90), break 83333.34 us (2048 delays = $800)
lit #$0
st TONE_PERIOD_2
lit #$C
st TONE_PERIOD_1
lit #$D
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$8
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$B
st RETURN_ADDR_0
jmp audio_play_tone

next43:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 291666.7 us (144 periods = $90), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$C
st RETURN_ADDR_0
jmp audio_play_tone

next44:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 291666.7 us (144 periods = $90), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$D
st RETURN_ADDR_0
jmp audio_play_tone

next45:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$E
st RETURN_ADDR_0
jmp rest_delay

next46:
; Play note 54 4F#, freq = 369.99 Hz, period 2703 us, TONE_PERIOD $88, duration 291666.7 us (108 periods = $6C), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$C
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$2
st RETURN_ADDR_1
lit #$F
st RETURN_ADDR_0
jmp audio_play_tone

next47:
; Play note 54 4F#, freq = 369.99 Hz, period 2703 us, TONE_PERIOD $88, duration 291666.7 us (108 periods = $6C), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$C
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$0
st RETURN_ADDR_0
jmp audio_play_tone

next48:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$1
st RETURN_ADDR_0
jmp rest_delay

next49:
; Play note 38 3D, freq = 146.83 Hz, period 6811 us, TONE_PERIOD $15A, duration 291666.7 us (43 periods = $2B), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$A
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$B
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$2
st RETURN_ADDR_0
jmp audio_play_tone

next50:
jmp part

middle:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$3
st RETURN_ADDR_0
jmp audio_play_tone

next51:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$4
st RETURN_ADDR_0
jmp audio_play_tone

next52:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$5
st RETURN_ADDR_0
jmp rest_delay

next53:
; Play note 38 3D, freq = 146.83 Hz, period 6811 us, TONE_PERIOD $15A, duration 291666.7 us (43 periods = $2B), break 41666.67 us (1024 delays = $400)
lit #$1
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$A
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$B
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$6
st RETURN_ADDR_0
jmp audio_play_tone

next54:
jmp part

conclusion: 
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 291666.7 us (144 periods = $90), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$7
st RETURN_ADDR_0
jmp audio_play_tone

next55:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 291666.7 us (144 periods = $90), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$9
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$8
st RETURN_ADDR_0
jmp audio_play_tone

next56:
; Rest duration 333333.3 us (8192 delays = $2000)
lit #$2
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$9
st RETURN_ADDR_0
jmp rest_delay

next57:
; Play note 52 4E, freq = 329.63 Hz, period 3034 us, TONE_PERIOD $99, duration 291666.7 us (96 periods = $60), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$9
st TONE_PERIOD_1
lit #$9
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$A
st RETURN_ADDR_0
jmp audio_play_tone

next58:
; Play note 52 4E, freq = 329.63 Hz, period 3034 us, TONE_PERIOD $99, duration 291666.7 us (96 periods = $60), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$9
st TONE_PERIOD_1
lit #$9
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$B
st RETURN_ADDR_0
jmp audio_play_tone

next59:
; Play note 55 4G, freq = 392.00 Hz, period 2552 us, TONE_PERIOD $80, duration 291666.7 us (114 periods = $72), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$0
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$7
st TONE_DURATION_1
lit #$2
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$C
st RETURN_ADDR_0
jmp audio_play_tone

next60:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 145833.3 us (72 periods = $48), break 20833.33 us (512 delays = $200)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$4
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$2
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$D
st RETURN_ADDR_0
jmp audio_play_tone

next61:
; Rest duration 166666.7 us (4096 delays = $1000)
lit #$1
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$E
st RETURN_ADDR_0
jmp rest_delay

next62:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 1166667 us (576 periods = $240), break 166666.7 us (4096 delays = $1000)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$2
st TONE_DURATION_2
lit #$4
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$1
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$3
st RETURN_ADDR_1
lit #$F
st RETURN_ADDR_0
jmp audio_play_tone

next63:
; Play note 56 4G#, freq = 415.30 Hz, period 2408 us, TONE_PERIOD $79, duration 291666.7 us (121 periods = $79), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$9
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$7
st TONE_DURATION_1
lit #$9
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$0
st RETURN_ADDR_0
jmp audio_play_tone

next64:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$1
st RETURN_ADDR_0
jmp audio_play_tone

next65:
; Play note 66 5F#, freq = 739.99 Hz, period 1352 us, TONE_PERIOD $43, duration 1166667 us (863 periods = $35F), break 166666.7 us (4096 delays = $1000)
lit #$0
st TONE_PERIOD_2
lit #$4
st TONE_PERIOD_1
lit #$3
st TONE_PERIOD_0
lit #$3
st TONE_DURATION_2
lit #$5
st TONE_DURATION_1
lit #$F
st TONE_DURATION_0
lit #$1
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$2
st RETURN_ADDR_0
jmp audio_play_tone

next66:
; Play note 62 5D, freq = 587.33 Hz, period 1703 us, TONE_PERIOD $55, duration 291666.7 us (171 periods = $AB), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$5
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$A
st TONE_DURATION_1
lit #$B
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$3
st RETURN_ADDR_0
jmp audio_play_tone

next67:
; Play note 54 4F#, freq = 369.99 Hz, period 2703 us, TONE_PERIOD $88, duration 291666.7 us (108 periods = $6C), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$C
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$4
st RETURN_ADDR_0
jmp audio_play_tone

next68:
; Play note 54 4F#, freq = 369.99 Hz, period 2703 us, TONE_PERIOD $88, duration 583333.3 us (216 periods = $D8), break 83333.34 us (2048 delays = $800)
lit #$0
st TONE_PERIOD_2
lit #$8
st TONE_PERIOD_1
lit #$8
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$D
st TONE_DURATION_1
lit #$8
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$8
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$5
st RETURN_ADDR_0
jmp audio_play_tone

next69:
; Play note 52 4E, freq = 329.63 Hz, period 3034 us, TONE_PERIOD $99, duration 291666.7 us (96 periods = $60), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$9
st TONE_PERIOD_1
lit #$9
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$6
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$6
st RETURN_ADDR_0
jmp audio_play_tone

next70:
; Play note 59 4B, freq = 493.88 Hz, period 2025 us, TONE_PERIOD $65, duration 583333.3 us (288 periods = $120), break 83333.34 us (2048 delays = $800)
lit #$0
st TONE_PERIOD_2
lit #$6
st TONE_PERIOD_1
lit #$5
st TONE_PERIOD_0
lit #$1
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$8
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$7
st RETURN_ADDR_0
jmp audio_play_tone

next71:
; Play note 57 4A, freq = 440.00 Hz, period 2273 us, TONE_PERIOD $72, duration 291666.7 us (128 periods = $80), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$7
st TONE_PERIOD_1
lit #$2
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$8
st TONE_DURATION_1
lit #$0
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$8
st RETURN_ADDR_0
jmp audio_play_tone

next72:
; Play note 50 4D, freq = 293.66 Hz, period 3406 us, TONE_PERIOD $AC, duration 291666.7 us (86 periods = $56), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$A
st TONE_PERIOD_1
lit #$C
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$5
st TONE_DURATION_1
lit #$6
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$9
st RETURN_ADDR_0
jmp audio_play_tone

next73:
; Rest duration 166666.7 us (4096 delays = $1000)
lit #$1
st DELAY_COUNTER_3
lit #$0
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$A
st RETURN_ADDR_0
jmp rest_delay

next74:
; Play note 50 4D, freq = 293.66 Hz, period 3406 us, TONE_PERIOD $AC, duration 145833.3 us (43 periods = $2B), break 20833.33 us (512 delays = $200)
lit #$0
st TONE_PERIOD_2
lit #$A
st TONE_PERIOD_1
lit #$C
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$2
st TONE_DURATION_1
lit #$B
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$2
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$B
st RETURN_ADDR_0
jmp audio_play_tone

next75:
; Play note 50 4D, freq = 293.66 Hz, period 3406 us, TONE_PERIOD $AC, duration 291666.7 us (86 periods = $56), break 41666.67 us (1024 delays = $400)
lit #$0
st TONE_PERIOD_2
lit #$A
st TONE_PERIOD_1
lit #$C
st TONE_PERIOD_0
lit #$0
st TONE_DURATION_2
lit #$5
st TONE_DURATION_1
lit #$6
st TONE_DURATION_0
lit #$0
st DELAY_COUNTER_3
lit #$4
st DELAY_COUNTER_2
lit #$0
st DELAY_COUNTER_1
lit #$0
st DELAY_COUNTER_0
lit #$4
st RETURN_ADDR_1
lit #$C
st RETURN_ADDR_0
jmp audio_play_tone

next76:
jmp halt

rest_delay_return:
ld RETURN_ADDR_1
jnz +
ld RETURN_ADDR_0
jz next0
cmpi #1
jz next1
cmpi #2
jz next2
cmpi #3
jz next3
cmpi #4
jz next4
cmpi #5
jz next5
cmpi #6
jz next6
cmpi #7
jz next7
cmpi #8
jz next8
cmpi #9
jz next9
cmpi #10
jz next10
cmpi #11
jz next11
cmpi #12
jz next12
cmpi #13
jz next13
cmpi #14
jz next14
jmp next15
+ cmpi #1
jnz +
ld RETURN_ADDR_0
jz next16
cmpi #1
jz next17
cmpi #2
jz next18
cmpi #3
jz next19
cmpi #4
jz next20
cmpi #5
jz next21
cmpi #6
jz next22
cmpi #7
jz next23
cmpi #8
jz next24
cmpi #9
jz next25
cmpi #10
jz next26
cmpi #11
jz next27
cmpi #12
jz next28
cmpi #13
jz next29
cmpi #14
jz next30
jmp next31
+ cmpi #2
jnz +
ld RETURN_ADDR_0
jz next32
cmpi #1
jz next33
cmpi #2
jz next34
cmpi #3
jz next35
cmpi #4
jz next36
cmpi #5
jz next37
cmpi #6
jz next38
cmpi #7
jz next39
cmpi #8
jz next40
cmpi #9
jz next41
cmpi #10
jz next42
cmpi #11
jz next43
cmpi #12
jz next44
cmpi #13
jz next45
cmpi #14
jz next46
jmp next47
+ cmpi #3
jnz +
ld RETURN_ADDR_0
jz next48
cmpi #1
jz next49
cmpi #2
jz next50
cmpi #3
jz next51
cmpi #4
jz next52
cmpi #5
jz next53
cmpi #6
jz next54
cmpi #7
jz next55
cmpi #8
jz next56
cmpi #9
jz next57
cmpi #10
jz next58
cmpi #11
jz next59
cmpi #12
jz next60
cmpi #13
jz next61
cmpi #14
jz next62
jmp next63
+ cmpi #4
jnz +
ld RETURN_ADDR_0
jz next64
cmpi #1
jz next65
cmpi #2
jz next66
cmpi #3
jz next67
cmpi #4
jz next68
cmpi #5
jz next69
cmpi #6
jz next70
cmpi #7
jz next71
cmpi #8
jz next72
cmpi #9
jz next73
cmpi #10
jz next74
cmpi #11
jz next75
cmpi #12
jz next76
; unknown return address
+ 
halt:
jmp halt

; ===== LCD ===== 

; use LCD_BUFFER_INDEX to get the next nibble from LCD_BUFFER, and put it in LCD_NIBBLE
lcd_write_buffer:
    ; value for LCD_BUFFER_INDEX is in the accumulator
    st LCD_BUFFER_INDEX
    
    jnz +
    ld LCD_BUFFER
    jmp lcd_write_nibble

+   cmpi #1
    jnz +
    ld LCD_BUFFER+1
    jmp lcd_write_nibble

+   cmpi #2
    jnz +
    ld LCD_BUFFER+2
    jmp lcd_write_nibble

+   cmpi #3
    jnz +
    ld LCD_BUFFER+3
    jmp lcd_write_nibble

+   cmpi #4
    jnz +
    ld LCD_BUFFER+4
    jmp lcd_write_nibble

+   cmpi #5
    jnz +
    ld LCD_BUFFER+5
    jmp lcd_write_nibble

+   cmpi #6
    jnz +
    ld LCD_BUFFER+6
    jmp lcd_write_nibble

+   cmpi #7
    jnz +
    ld LCD_BUFFER+7
    jmp lcd_write_nibble

+   cmpi #8
    jnz +
    ld LCD_BUFFER+8
    jmp lcd_write_nibble

+   cmpi #9
    jnz +
    ld LCD_BUFFER+9
    jmp lcd_write_nibble

+   cmpi #10
    jnz +
    ld LCD_BUFFER+10
    jmp lcd_write_nibble

+   cmpi #11
    jnz +
    ld LCD_BUFFER+11
    jmp lcd_write_nibble

+   cmpi #12
    jnz +
    ld LCD_BUFFER+12
    jmp lcd_write_nibble

+   cmpi #13
    jnz +
    ld LCD_BUFFER+13
    jmp lcd_write_nibble

+   cmpi #14
    jnz +
    ld LCD_BUFFER+14
    jmp lcd_write_nibble

+   ld LCD_BUFFER+15

lcd_write_nibble:
    out #PORT_LCD
    ld LCD_CONTROL_STATE
    out #PORT_CONTROL ; setup RS
    _ori #1 ; set bit 1 (E)
    out #PORT_CONTROL
    _andi #$E ; clear bit 1 (E)
    out #PORT_CONTROL

    ; wait at least LCD_SINGLE_COMMAND_DELAY_US us for the LCD
    ; each loop iteration is 6 clocks
    ; initial count = round_up(LCD_SINGLE_COMMAND_DELAY_US * CPU_CLOCKS_PER_US / 6) - 1
    ; Initial count of 15 = 16 iterations = 96 clocks = 39 us @ 2.4576 MHz.
    lit #15
-   addi #0 ; NOP
    addi #-1
    jc -    ; carry will be clear when result goes negative

    ; decrement the buffer index
    ld LCD_BUFFER_INDEX
    addi #-1
    jc lcd_write_buffer

    ; determine the return address
    ld TEXT_RETURN_ADDR
    jz textnext0
    cmpi #1
    jz textnext1
    cmpi #2
    jz textnext2
    cmpi #3
    jz textnext3
    cmpi #4
    jz textnext4
    cmpi #5
    jz textnext5
    cmpi #6
    jz textnext6

    ; unknown return address
    jmp halt

lcd_long_delay:
    ; wait at least LCD_CLEAR_HOME_DELAY_US us for the LCD
    ; each loop iteration is 6 clocks
    ; initial count = round_up(LCD_CLEAR_HOME_DELAY_US * CPU_CLOCKS_PER_US / 6) - 1
    ; Initial count of 656 = $290 hex = 657 iterations = 3942 clocks = 1604 us @ 2.4576 MHz
    lit #2
    st LCD_DELAY_2
    lit #9
    st LCD_DELAY_1
    lit #0
-   addi #0 ; NOP
    addi #-1
    jc -    ; carry will be clear when result goes negative
    ld LCD_DELAY_1
    addi #-1
    st LCD_DELAY_1
    jnc +
    lit #$F
    jmp -
+   ld LCD_DELAY_2
    addi #-1
    st LCD_DELAY_2
    jnc +
    lit #$F
    jmp -
+   ; end of delay loop

    ; determine the return address
    ld RETURN_ADDR_0
    jz delaynext0
    cmpi #1
    jz delaynext1
    cmpi #2
    jz delaynext2

    ; unknown return address
    jmp halt
